/*
 * linux/arch/arm/mach-at91rm9200/board-unc90.c
 *
 *  Copyright (c) 2005 Sistemas Embebidos S.A.
 *                     A Digi International Company.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/device.h>

#include <asm/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/arch/hardware.h>
#include <asm/mach/serial_at91rm9200.h>
#include <asm/arch/board.h>
#include <asm/arch/unc90utils.h>
#include <asm/arch/gpio.h>
#include "generic.h"

static void __init unc90_init_irq(void)
{
	/* Initialize AIC controller */
	at91rm9200_init_irq(NULL);

	/* Set up the GPIO interrupts */
	at91_gpio_irq_setup(BGA_GPIO_BANKS);
}

/***********************************************************************
 * @Function: at91rm9200_config_smc_cs
 * @Return:   0 on success, other value on error.
 * @Descr:    Configures the Static Memory Controller Chip Select
 *            register.
 ***********************************************************************/
static int at91rm9200_config_smc_cs( int cs, int wait_states,
                                     int waits_enable, int tdf,
                                     int bat, int dbw, int drp,
                                     int acss, int rwsetup, int rwhold )
{
	unsigned int mask;

	// Check parameters
	if( ( (cs < 0) || (cs > 7) ) &&
            ( (wait_states < 0) || (wait_states > 127) ) &&
            ( (tdf < 0) || (tdf > 127) ) &&
            ( (dbw < 0) || (dbw > 2) ) &&
            ( (acss < 0) || (acss > 3) ) &&
            ( (rwsetup < 0) || (rwsetup > 7) ) &&
            ( (rwhold < 0) || (rwhold > 7) ) )
	{
	    return -1;
	}

	// Configure GPIOs, if necessary
	if( cs > 3 )
	{
	    switch( cs )
	    {
#if 0
	        case 4:	mask = AT91_PC10_NCS4_CFCS;	break;
	        case 5:	mask = AT91_PC11_NCS5_CFCE1;	break;
	        case 6:	mask = AT91_PC12_NCS6_CFCE2;	break;
	        case 7:	mask = AT91_PC13_NCS7;		break;
	        default: mask = 0;
#endif
	        case 4:	mask = AT91_PIN_PC10;	break;
	        case 5:	mask = AT91_PIN_PC11;	break;
	        case 6:	mask = AT91_PIN_PC12;	break;
	        case 7:	mask = AT91_PIN_PC13;	break;
	        default: mask = 0;
	    }
	    
	    at91_set_A_periph( mask, 0 );
	}

	at91_sys_write( AT91_SMC_CSR(mask), 
		(AT91_SMC_NWS & wait_states) |
		(waits_enable ? AT91_SMC_WSEN : 0) |
		((0x0f & tdf) << 8) | (bat ? AT91_SMC_BAT : 0) |
		((0x03 & dbw) << 13) | (drp ? AT91_SMC_DPR : 0) |
		((0x03 & acss) << 16) | ((0x07 & rwsetup) << 24) |
		((0x07 & rwhold) << 28) );


#if 0
	// Write to SMC Chip Select register the new configuration
	AT91_SYS->EBI_SMC2_CSR[cs] = (AT91_SMC2_NWS & wait_states) |
	                             (waits_enable ? AT91_SMC2_WSEN : 0) |
                                     ((0x0f & tdf) << 8) | (bat ? AT91_SMC2_BAT : 0) |
	                             ((0x03 & dbw) << 13) | (drp ? AT91_SMC2_DRP : 0) |
	                             ((0x03 & acss) << 16) | ((0x07 & rwsetup) << 24) |
	                             ((0x07 & rwhold) << 28);
#endif


	return 0;
}

/***********************************************************************
 * @Function: unc90_config_smc_cs
 * @Return:   0 on success, other value on error.
 * @Descr:    Configures the CS signals CS3 and CS4.
 ***********************************************************************/
int unc90_config_smc_cs( int cs, int wait_states,
                         int waits_enable, int tdf,
                         int bat, int dbw, int drp,
                         int acss, int rwsetup, int rwhold )
{
	// Check parameters
	if( ( (cs < 3) || (cs > 4) ) )
	    return -1;

	return at91rm9200_config_smc_cs( cs, wait_states, waits_enable,
                                         tdf, bat, dbw, drp, acss,
                                         rwsetup, rwhold );
}
EXPORT_SYMBOL( unc90_config_smc_cs );



/*
 * Serial port configuration.
 *    0 .. 3 = USART0 .. USART3
 *    4      = DBGU
 * On UNC90 module UNC90 PORTA is considered CONFIG_UNC90_ENABLE_SERIAL_1, which corresponds
 * with AT91RM5200 USART1.
 * UNC90 PORTC is considered CONFIG_UNC90_ENABLE_SERIAL_2, which corresponds with AT91RM9200
 * USART2.
 * It is also available the Debug Unit Port on the serial connector, over the hardware
 * flow control signals. To use this port, with the base board XXXXX, refer to XXXXX.
 * UNC90_UART_MAP { X, X, X, X, X }  ttyS0, ..., ttyS4
 */

#ifdef CONFIG_SERIAL_AT91_CONSOLE
# if defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
     defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
     defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 4, 1, 2, -1, -1 }
# elif  defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
        defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
        !defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 1, 2, -1, -1, -1 }
# elif  defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
        !defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
        defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 4, 1, -1, -1, -1 }
# elif  defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
        !defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
        !defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 1, -1, -1, -1, -1 }
# elif  !defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
        defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
        defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 4, 2, -1, -1, -1 }
# elif  !defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
        defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
        !defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 2, -1, -1, -1, -1 }
# elif  !defined(CONFIG_UNC90_ENABLE_SERIAL_1) && \
        !defined(CONFIG_UNC90_ENABLE_SERIAL_2) && \
        defined(CONFIG_UNC90_ENABLE_SERIAL_DBG)
#  define	UNC90_UART_MAP	{ 4, -1, -1, -1, -1 }
# else // No serial port defined!
#  warning UNC90 no serial port defined
# endif
#else  //CONFIG_SERIAL_AT91_CONSOLE
# define	UNC90_UART_MAP	{ -1, -1, -1, -1, -1 }
#endif //CONFIG_SERIAL_AT91_CONSOLE

#if defined( CONFIG_UNC90_CONSOLE_ON_PORTC ) && defined( CONFIG_UNC90_ENABLE_SERIAL_2 )
# if defined( CONFIG_UNC90_ENABLE_SERIAL_DBG ) && defined(CONFIG_UNC90_ENABLE_SERIAL_1)
#  define	UNC90_SERIAL_CONSOLE	2
# elif !defined( CONFIG_UNC90_ENABLE_SERIAL_DBG ) && !defined(CONFIG_UNC90_ENABLE_SERIAL_1)
#  define	UNC90_SERIAL_CONSOLE	0
# else
#  define	UNC90_SERIAL_CONSOLE	1
# endif
#elif	defined( CONFIG_UNC90_CONSOLE_ON_PORTA ) && defined( CONFIG_UNC90_ENABLE_SERIAL_1 )
# if defined( CONFIG_UNC90_ENABLE_SERIAL_DBG )
#  define	UNC90_SERIAL_CONSOLE	1
# else
#  define	UNC90_SERIAL_CONSOLE	0
# endif
#elif	defined( CONFIG_UNC90_CONSOLE_ON_DEBUG ) && defined( CONFIG_UNC90_ENABLE_SERIAL_DBG )
# define	UNC90_SERIAL_CONSOLE	0
#else
# error UNC90 Console is configured on a disabled Serial Port
#endif


static void __init unc90_map_io(void)
{
	int serial[AT91_NR_UART] = UNC90_UART_MAP;
	int i;
	
	at91rm9200_map_io();

	/* Initialize clocks: 18.432 MHz crystal */
	at91_clock_init(18432000);

#ifdef CONFIG_SERIAL_AT91
	at91_console_port = UNC90_SERIAL_CONSOLE;
	memcpy(at91_serial_map, serial, sizeof(serial));

	/* Register UARTs */
	for (i = 0; i < AT91_NR_UART; i++) {
		if (at91_serial_map[i] >= 0)
			at91_register_uart(i, at91_serial_map[i]);
	}
#endif
}

static struct at91_eth_data __initdata unc90_eth_data = {
	.phy_irq_pin	= AT91_PIN_PC4,
	.is_rmii	= 0,
};

#if defined(CONFIG_USB_OHCI_HCD) && defined(CONFIG_USB_GADGET)
# error On UNC90 its not possible to use USB Host & Device simultaneously
# error Disable one or build it as modules.
#endif

#if defined(CONFIG_USB_OHCI_HCD) || defined(CONFIG_USB_OHCI_HCD_MODULE)
static struct at91_usbh_data __initdata unc90_usbh_data = {
	.ports		= 1,
};
#endif


#if defined(CONFIG_USB_GADGET_AT91)
static struct at91_udc_data __initdata unc90_udc_data = {
	.vbus_pin	= -1,	/* Not used on UNC90 */
	.pullup_pin	= -1,	/* Not used on UNC90 */
};
#endif

static void __init unc90_board_init(void)
{
	/* Ethernet */
	at91_add_device_eth(&unc90_eth_data);

#if defined(CONFIG_USB_OHCI_HCD) || defined(CONFIG_USB_OHCI_HCD_MODULE)
	/* USB Host */
	at91_add_device_usbh(&unc90_usbh_data);
#endif

#if defined(CONFIG_USB_GADGET_AT91)
	/* USB Device */
	at91_add_device_udc(&unc90_udc_data);
#endif
}

MACHINE_START(UNC90, "UNC90")
	/* Maintainer: Sistemas Embebidos SA, a Digi International Company */
	.phys_ram	= AT91_SDRAM_BASE,
	.phys_io	= AT91_BASE_SYS,
	.io_pg_offst	= (AT91_VA_BASE_SYS >> 18) & 0xfffc,
	.boot_params	= AT91_SDRAM_BASE + 0x100,
	.timer		= &at91rm9200_timer,
	.map_io		= unc90_map_io,
	.init_irq	= unc90_init_irq,
	.init_machine	= unc90_board_init,
MACHINE_END
