/*
 * drivers/net/phy/ics.c
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/mii.h>
#include <linux/ethtool.h>
#include <linux/phy.h>		/* phy_driver */

static int __init ics_init( void );
static void __exit ics_exit( void );

static struct phy_driver ics1893_driver = {
	.phy_id		= 0x0015f440,
	.name		= "ICS1893",
	.phy_id_mask	= 0x00fffff0,
	.features	= PHY_BASIC_FEATURES,
	.config_aneg	= genphy_config_aneg,
	.read_status	= genphy_read_status,
	.driver 	= {
		.owner = THIS_MODULE,
	},
};

static int __init ics_init( void )
{
	int ret;

	ret = phy_driver_register( &ics1893_driver );

	return ret;
}

static void __exit ics_exit( void )
{
	phy_driver_unregister( &ics1893_driver );
}

module_init( ics_init );
module_exit( ics_exit );

MODULE_DESCRIPTION( "ICS 1893 driver" );
MODULE_AUTHOR( "Markus Pietrek" );
MODULE_LICENSE( "GPL" );
