/***********************************************************************
 *
 *  Copyright 2005 by Sistemas Embebidos S.A.
 *  All rights reserved.
 *
 *  $Id: unc90_gpio.h,v 1.1 2005-09-27 08:02:40 pperez Exp $
 *  Author: Pedro Perez de Heredia
 *  Descr:  UNC90 GPIO driver. 
 *  
 ***********************************************************************
 *  @History:
 *     2005/04/11 : Initial version.
 ***********************************************************************/

#ifndef	_UNC90_GPIO_H_
#define _UNC90_GPIO_H_

#ifdef __KERNEL__

#include <asm/arch/hardware.h>

#ifndef UNC90_GPIO_MAJOR
 #define UNC90_GPIO_MAJOR	0	// dynamic major
#endif

#ifndef UNC90_GPIO_MAX_PORTS
 #define UNC90_GPIO_MAX_PORTS	2	// max number of GPIO port handlers.
#endif

#define	UNC90_GPIO_LINES_X_PORT	8	// gpio lines per port 
#define	UNC90_MAX_GPIOS		(UNC90_GPIO_LINES_X_PORT*UNC90_GPIO_MAX_PORTS)



// Input flags
#define GPIO_INP_GLITCH_FILTER	0x01
#define GPIO_PULL_UP		0x02


typedef enum
{
	GPIO_UNCONFIG,
	GPIO_INPUT,
	GPIO_OUTPUT,
	GPIO_IRQ
}unc90_gpio_t;

typedef enum
{
	PIO_A = 0,
	PIO_B,
	PIO_C,
	PIO_D
}pio_t;

typedef struct 
{
	unc90_gpio_t 	mode;
	unsigned int	flags;
	pio_t		port;
	int		bit;
	
}unc90_gpio_conf_t;

unc90_gpio_conf_t unc90_porta_default_conf[] = {
	
/* PORTA0 */	{ GPIO_UNCONFIG,          0,    PIO_B,     0 },	/* PB0 and PB23 */ 
/* PORTA1 */	{ GPIO_UNCONFIG,          0,    PIO_B,    24 },	/* PB24 */ 
/* PORTA2 */	{ GPIO_UNCONFIG,          0,    PIO_B,    25 },	/* PB25 */ 
/* PORTA3 */	{ GPIO_UNCONFIG,          0,    PIO_B,     3 },	/* PB3 and PB21 */ 
/* PORTA4 */	{ GPIO_UNCONFIG,          0,    PIO_B,     1 },	/* PB1 and PA2 */
/* PORTA5 */	{ GPIO_UNCONFIG,          0,    PIO_B,    26 },	/* PB26 */
///* PORTA6 */	{ GPIO_OUTPUT,            0,    PIO_D,    25 },	/* PD25 */
/* PORTA6 */	{ GPIO_UNCONFIG,          0,    PIO_D,    25 },	/* PD25 */
/* PORTA7 */	{ GPIO_UNCONFIG,          0,    PIO_B,     2 },	/* PB2 and PB20 */

};

unc90_gpio_conf_t unc90_portc_default_conf[] = {
	
/* PORTC0 */	{ GPIO_UNCONFIG,          0,    PIO_B,     6 },	/* PB6 and PB29 */
///* PORTC0 */	{ GPIO_IRQ,               0,    PIO_B,     6 },	/* PB6 and PB29 */
/* PORTC1 */	{ GPIO_UNCONFIG,          0,    PIO_A,    30 },	/* PA30 */
/* PORTC2 */	{ GPIO_UNCONFIG,          0,    PIO_A,    25 },	/* PA25 */
/* PORTC3 */	{ GPIO_UNCONFIG,          0,    PIO_B,     9 },	/* PA22 and PB9 */
/* PORTC4 */	{ GPIO_UNCONFIG,          0,    PIO_B,     7 },	/* PA24 and PB7 */
///* PORTC4 */	{ GPIO_IRQ,               0,    PIO_B,     7 },	/* PA24 and PB7 */
/* PORTC5 */	{ GPIO_UNCONFIG,          0,    PIO_A,    31 },	/* PA31 */
/* PORTC6 */	{ GPIO_UNCONFIG,          0,    PIO_A,    26 },	/* PA26 */
/* PORTC7 */	{ GPIO_UNCONFIG,          0,    PIO_B,     8 },	/* PA23 and PB8 */

};

#endif // __KERNEL__


// ioctl magic numbers
#define GPIO_IOCTL_BASE		'G'

// inputs
#define GPIO_CONFIG_AS_INP	_IOR (GPIO_IOCTL_BASE, 0, int)	/* config this pin as input */
#define GPIO_READ_PIN_VAL	_IOR (GPIO_IOCTL_BASE, 1, int)	/* read pin value */
#define GPIO_SET_INP_FILTER	_IOR (GPIO_IOCTL_BASE, 2, int)	/* input filter pin value */

// outputs
#define GPIO_CONFIG_AS_OUT	_IOR (GPIO_IOCTL_BASE, 5, int)	/* config this pin as output */
#define GPIO_WRITE_PIN_VAL	_IOW (GPIO_IOCTL_BASE, 6, int)	/* sets the pin value */
#define GPIO_CONFIG_PULL_UP	_IOW (GPIO_IOCTL_BASE, 7, int)	/* sets the pull-up value */

// irqs
#define GPIO_CONFIG_AS_IRQ	_IOR (GPIO_IOCTL_BASE, 10, int)	/* config this pin as interrupt */



#endif // _UNC90_GPIO_H_

