/*
 * include/asm-arm/arch-ns9xxx/regs-bbus.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_REGSBBUS_BRIDGE_H
#define __ASM_ARCH_REGSBBUS_BRIDGE_H

#include <asm/arch/hardware.h>

/* BBus Bridge */

/* DMA Channel 1 Buffer Descriptor Pointer */
#define BBUS_DMA1BD	__REG(0xa0400000)

/* DMA Channel 1 Control register */
#define BBUS_DMA1CR	__REG(0xa0400004)

/* DMA Channel 1 Status and Interrupt Enable */
#define BBUS_DMA1SIE	__REG(0xa0400008)

/* DMA Channel 1 Peripheral Chip Select */
#define BBUS_DMA1PCS	__REG(0xa040000C)

/* DMA Channel 2 Buffer Descriptor Pointer */
#define BBUS_DMA2BD	__REG(0xa0400020)

/* DMA Channel 2 Control register */
#define BBUS_DMA2CR	__REG(0xa0400024)

/* DMA Channel 2 Status and Interrupt Enable */
#define BBUS_DMA2SIE	__REG(0xa0400028)

/* DMA Channel 2 Peripheral Chip Select */
#define BBUS_DMA2PCS	__REG(0xa040002C)

/* BBus Bridge Interrupt Status register */
#define BBUS_ISR	__REG(0xa0401000)

/* BBus Bridge Interrupt Enable register */
#define BBUS_IER	__REG(0xa0401004)

/* Enable the aggregate interrupt signal to propagate
   to the VIC in the System Control module */
#define BBUS_IER_GLBL	__REGBIT(31)

/* Enable interrupt from AHB DMA Channel #2 */
#define BBUS_IER_DMA2	__REGBIT(25)

/* Enable interrupt from AHB DMA Channel #1 */
#define BBUS_IER_DMA1	__REGBIT(24)

/* Enable interrupt from USB Device */
#define BBUS_IER_USBDE	__REGBIT(15)

/* Enable interrupt from USB Host */
#define BBUS_IER_USBHE	__REGBIT(14)

/* Enable interrupt from RTC module */
#define BBUS_IER_RTCE	__REGBIT(13)

/* Enable interrupt from BBUS Utility module */
#define BBUS_IER_UTILE	__REGBIT(12)

/* Enable interrupt from IEEE-1284 module */
#define BBUS_IER_1284E	__REGBIT(11)

/* Enable interrupt from I2C module */
#define BBUS_IER_I2CE	__REGBIT(10)

/* Enable interrupt from SER transmit module D */
#define BBUS_IER_SDTXE	__REGBIT(9)

/* Enable interrupt from SER receive module D */
#define BBUS_IER_SDRXE	__REGBIT(8)

/* Enable interrupt from SER transmit module C */
#define BBUS_IER_SCTXE	__REGBIT(7)

/* Enable interrupt from SER receive module C */
#define BBUS_IER_SCRXE	__REGBIT(6)

/* Enable interrupt from SER transmit module A */
#define BBUS_IER_SATXE	__REGBIT(5)

/* Enable interrupt from SER receive module A */
#define BBUS_IER_SARXE	__REGBIT(4)

/* Enable interrupt from SER transmit module B */
#define BBUS_IER_SBTXE	__REGBIT(3)

/* Enable interrupt from SER receive module B */
#define BBUS_IER_SBRXE	__REGBIT(2)

/* Enable interrupt from USB module */
#define BBUS_IER_USBE	__REGBIT(1)

/* Enable aggregate interrupt from BBus DMA
   module. These interrupts can be controlled on a
   per-DMA channel basis in the BBus utility module. */
#define BBUS_IER_DMAE	__REGBIT(0)

#endif /* ifndef __ASM_ARCH_REGSBBUS_BRIDGE_H */
