/*
 * Copyright (C) 2005 by FS Forth-Systeme GmbH.
 * All rights reserved.
 * Jonas Dietsche <jdietsche@fsforth.de>
 *
 * Configuation settings for the FS Forth-Systeme GmbH's A9M9750 Module
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <asm-arm/sizes.h>
#include <ns9750_sys.h>

#include <configs/digi_common.h>

#define CONFIG_IDENT_STRING	" " VERSION_TAG "\nfor Digi " MODULE_STRING " on ValiBoard"

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_ARM926EJS	1	/* This is an ARM926EJS Core	*/
#define CONFIG_NS9360		1	/* in an NetSilicon NS9360 SoC     */
#define CONFIG_A9M9360		1       /* on an FS Forth-Systeme Module */
#define BOARD_LATE_INIT		1	/* Enables initializations before jumping to main loop */
#define CPU     		"NS9360"

#define CONFIG_BOOT_NAND	1
#define NS9750_ETH_PHY_ADDRESS	(0x0001)
#define CONFIG_SYS_CLK_FREQ	353894400

#define CPU_CLK_FREQ		(CONFIG_SYS_CLK_FREQ/2)
#define AHB_CLK_FREQ		(CONFIG_SYS_CLK_FREQ/4)
#define BBUS_CLK_FREQ		(CONFIG_SYS_CLK_FREQ/8)

#undef CONFIG_USE_IRQ           /* we don't need IRQ/FIQ stuff */

#define CONFIG_GCC41	1       /* define for compiling with GCC 4.1 */

/*
 * Hardware drivers
 */
#define CFG_NO_FLASH		1 /* No NOR-Flash available */

 /* USB related stuff */
#define LITTLEENDIAN		1       /* necessary for USB */
#define CONFIG_USB_OHCI 	1
#define CONFIG_DOS_PARTITION 	1
#define CONFIG_USB_STORAGE 	1
#define CONFIG_SILENT_RESCUE	1

/* ethernet stuff */
#define CONFIG_DRIVER_NS9750_ETHERNET	1 /* use on-chip ethernet */

#define CONFIG_MACH_A9M9360     /* Select board mach-type */

#ifdef CONFIG_A9MVALI		/* this option is passed in the Makefile */
# define CONFIG_MACH_CC9P9360VAL

/*
 * select serial console configuration
 */
# define CFG_NS9750_UART	1	/* use on-chip UART */
# define PLATFORM 		"val"
# define SER 			"ttyS1"
# define CONFIG_BOOTFILE	"uImage-cc9p9360val"

#elif CONFIG_A9M9750DEV

# define CONFIG_MACH_CC9P9360DEV

# define CFG_NS16550
# define CFG_NS16550_SERIAL
# define CFG_NS16550_REG_SIZE	 1
# define CFG_NS16550_CLK	 18432000
# define CFG_NS16550_COM1	 0x40000000
# define PLATFORM 		"dev"
# define SER 			"ttyS0"
# define CONFIG_BOOTFILE	"uImage-cc9p9360dev"
#endif /*CONFIG_A9MVALI*/

#define CONFIG_COMMANDS ( \
                CONFIG_COMMANDS_DIGI |          \
                CFG_CMD_NAND    |               \
                0 )

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>


/************************************************************
 * A9M9360 default environment settings
 ************************************************************/

#define CONFIG_EXTRA_ENV_SETTINGS			\
	"std_bootarg=console="SER",38400\0"		\
        "silent="CFG_SET_SILENT"\0"                     \
	"uimg=u-boot-cc9p9360"PLATFORM".bin\0"		\
	"rimg=rootfs-cc9p9360"PLATFORM".jffs2\0"	\
	"kimg=uImage-cc9p9360"PLATFORM"\0"		\
	"eimg=eboot-cc9p9360"PLATFORM"\0"	        \
	"wimg=wce-cc9p9360"PLATFORM"\0"                 \
	"loadaddr=200000\0"				\
	"ebootaddr=200000\0"			        \
	"ebootfladdr=40000\0"			       	\
	"wcefladdr=100000\0"				\
	"npath=/exports/nfsroot-cc9p9360"PLATFORM"\0"	\
	"smtd=setenv bootargs ${std_bootarg} ip=${ipaddr}:${serverip}::${netmask}:9360:eth0:off root=/dev/mtdblock2 rootfstype=jffs2\0"	\
	"snfs=setenv bootargs ${std_bootarg} ip=${ipaddr}:${serverip}::${netmask}:9360:eth0:off root=nfs nfsroot=${serverip}:${npath}\0"


/*
 * Miscellaneous configurable options
 */
#define	CFG_PROMPT		"CC9P9360 # "	/* Monitor Command Prompt */
#define	MODULE_STRING		"CC9P9360"

#define CFG_MEMTEST_START	0x00000000	/* memtest works on	*/
#define CFG_MEMTEST_END		0x00f80000	/* 15,5 MB in DRAM	*/

#undef  CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */

#define	CFG_LOAD_ADDR		0x00200000	/* default load address	*/

#define	CFG_HZ			(CPU_CLK_FREQ/64)

/* valid baudrates */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
/* TODO */
#define CONFIG_NR_DRAM_BANKS	1	   /* we have 1 bank of DRAM */
#define PHYS_SDRAM_1		0x00000000	 /* SDRAM Bank #1 */
/* size will be defined in include/configs when you make the config */
/*#define PHYS_SDRAM_1_SIZE */

#define PHYS_FLASH_1		0x50000000 /* Flash Bank #1 */
#define CFG_FLASH_BASE		PHYS_FLASH_1

#define PHYS_NAND_FLASH		0x50000000 /* NAND Flash Bank #1 */
#define CFG_NAND_BASE		PHYS_NAND_FLASH

#define NAND_BUSY_GPIO		49

/* taken from a9m2410 */

#define	CONFIG_RTC_DS1337	1
#define CFG_I2C_RTC_ADDR	0x68	/* Dallas DS1337 RTC address */

#define CONFIG_HARD_I2C		1	/* I2C with hardware support */
#define CONFIG_DRIVER_NS9750_I2C 1	/* I2C driver */
#define CFG_I2C_SPEED 		100000	/* Hz */
#define CFG_I2C_SLAVE 		0x7F	/* I2C slave addr */

#define CFG_I2C_EEPROM_ADDR	0x50	/* EEPROM address */
#define CFG_I2C_EEPROM_ADDR_LEN	2	/* 2 address bytes */

#undef CFG_I2C_EEPROM_ADDR_OVERFLOW
#define CFG_EEPROM_PAGE_WRITE_BITS 5	/* 32 bytes page write mode on M24LC64 */
#define CFG_EEPROM_PAGE_WRITE_DELAY_MS 10
#define CFG_EEPROM_PAGE_WRITE_ENABLE 1

#define CFG_ENV_IS_IN_DIGI_NVRAM	1
#define CFG_ENV_SIZE		0x00002000 /* some space for U-Boot */


/* they are calculated for 128kB sectors. So Kernel/FPGA starts at 1MB */
# define NAND_UBOOT_SIZE	0x000c0000
# define NAND_NVRAM_SIZE        0x00040000
# define NAND_KERNEL_SIZE       0x00300000
# define NAND_ROOTFS_SIZE       0x01000000

#define CONFIG_JFFS2_DEV	"nand0"
#define CONFIG_JFFS2_NAND

#if (CONFIG_COMMANDS & CFG_CMD_NAND)
# include <ns9750_nand.h>
#endif /* CONFIG_CMD_NAND */

#include <configs/digi_common_post.h>

#endif	/* __CONFIG_H */
